/*Program to convert NANs in the FLUX array to another user specified number*/

/*The code was written and debugged by John C. Martin at the University of
  Minnesota as part of the Eta Carinae HST Treasury Project (copyright
  2004,2005).  You are free to distribute this source as long as this
  notice remains in tact.  End users of this code agree to take
  responsibility for how this code is used and John C. Martin, the Eta
  Carinae HST Treasury Project, and the University of Minnesota bear
  no liability.*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "fitsio.h"

int main(int argc, char *argv[])
{
  fitsfile *fptr;
  char *fln,*cmd;
  float f1;
  double num,*test;
  double dnan=-999./0.;
  int sl,rz,x,y,type;
  int naxis;
  long naxes[3];
  int hdu=2; //HDU for FLUX array
  int status=0;
  int debug=1;

  fln=(char *)malloc(sizeof(char)*200);
  cmd=(char*)malloc(sizeof(char)*200);

  if (argc<=2) {
    printf("usage:  replacenan <input file> <number>\n   Program replaces all the instaces of NAN in the flux array of the input\n    file with the number specified.\n");
    return-1;
  }

  strcpy(fln,argv[1]);
  if (sscanf(argv[2],"%f",&f1)<=0) {
    printf("ERROR:  %s is not a valid floating point number\n",argv[2]);
    return(-1);
  }
  num=f1;

  if (debug) printf("Processing:  %s\n",fln);
  /*but first gunzip if zipped*/
  sl=strlen(fln);
  if (strcmp(fln+(sl-3),".gz")==0) {
    printf("Unzipping file....");
    sprintf(cmd,"gunzip %s",fln);
    system(cmd);
    fln[sl-3]=(char)NULL;
    printf(" .. done  ");
    rz=1;
  } else rz=0;

  /*Open the file and read variables*/
  if (ffopen(&fptr,fln,READWRITE,&status)) {
    ffrprt(stderr,status);
    return(-2);
  }
  ffmahd(fptr,hdu,&type,&status);
  ffgidm(fptr,&naxis,&status);
  ffgisz(fptr,3,naxes,&status);
  test=(double*)malloc(sizeof(double)*naxes[0]);
  /*Parse through flux array and replace NANs*/
  for (y=0;y<naxes[1];y++) {
    ffgpv(fptr,TDOUBLE,(y*naxes[0]+1),naxes[0],&dnan,test,&sl,&status);
    for (x=0;x<naxes[0];x++) {
      //      printf("%i %e",x,test[x]);
      if (test[x]==dnan) test[x]=num;
      //      printf(" %e\n",test[x]);
    }
    ffppr(fptr,TDOUBLE,(y*naxes[0]+1),naxes[0],test,&status);
  }
  ffclos(fptr,&status);
  if (status!=0) ffrprt(stderr,status);

  if (rz>0) {
    printf("  Zipping...");
    sprintf(cmd,"gzip %s",fln);
    system(cmd);
    printf("Done  \n");
  }

  return(0);
}
